package Pstax::GtkAutoDialog;

use strict;

use base qw(Class::Accessor::Fast);

__PACKAGE__->mk_accessors(qw(
  gearview
));

use Glib qw(TRUE FALSE);
use Gtk2::GladeXML;

my %widget_class_map = (
  'Gtk2::Entry'       => 'Pstax::GtkAutoDialog::Entry',
  'Gtk2::CheckButton' => 'Pstax::GtkAutoDialog::CheckButton',
  'Gtk2::TextView'    => 'Pstax::GtkAutoDialog::TextView',
);


sub new {
  my $class = shift;

  return bless { inputs => {}, @_ }, $class;
}

sub app { return shift->gearview->app; }


sub invoke {
  my $class = shift;
  my $self  = $class->new(@_);

  my $dialog = $self->build_dialog || return;

  if($dialog->run eq 'ok') {
    $self->save
  }

  $dialog->destroy;

  return 1;
}


sub save      { $_->save foreach (values %{$_[0]->{inputs}}); }

sub add_input { $_[0]->{inputs}->{$_[1]} = $_[2]; }
sub input     { $_[0]->{inputs}->{$_[1]};         }


sub build_dialog {
  my $self = shift;

  my $gearview = $self->gearview                               || return;
  my $xml      = $gearview->dialog_xml                         || return;
  my $gladexml = Gtk2::GladeXML->new_from_buffer($xml)         || return;

  my $dialog   = $gladexml->get_widget($gearview->dialog_name);
  if(!$dialog) {
    $self->app->alert(
      'Error loading properties dialog',
      "Can't find '" . $gearview->dialog_name . "' in:\n\n$xml"
    );
    return;
  }

  $self->find_inputs($dialog);

  return $dialog;
}


sub find_inputs {
  my($self, $widget) = @_;

  my $name = $widget->get_widget_name;
  if($name  and  $name =~ /^PAD\.(\w+)(?:\((.*?)\))?/) {
    if($2) {
      $self->connect_behaviour($widget, $1, $2);
    }
    else {
      $self->connect_property($widget, $1);
    }
  }
  return unless $widget->can('get_children');
  my @children = $widget->get_children;
  foreach my $child (@children) {
    $self->find_inputs($child);
  }
}


sub connect_property {
  my($self, $widget, $name) = @_;

  my $gear = $self->gearview->gear;
  return unless($gear->can($name));
  my $type = ref $widget;
  if(my $class = $widget_class_map{$type}) {
    $self->add_input(
      $name => $class->new(name => $name, widget => $widget, gear => $gear)
    )
  }
}


sub connect_behaviour {
  my($self, $widget, $behaviour, $args) = @_;

  if($behaviour eq 'browse_to_entry') {
    my $target = $args;
    $widget->signal_connect(clicked => sub {
      my $file_chooser = Gtk2::FileChooserDialog->new(
        'Open',
        undef,
        'open',
        'gtk-cancel' => 'cancel',
        'gtk-ok'     => 'ok'
      );
      if($file_chooser->run eq 'ok') {
        my $filename = $file_chooser->get_filename;
        $self->input($target)->set($filename);
      }
      $file_chooser->destroy;
    });
  }
  else {
    warn __PACKAGE__ . " Unrecognised behaviour: $behaviour($args)\n";
  }
}


package Pstax::GtkAutoDialog::Entry;


sub new {
  my $class = shift;

  my $self = bless { @_ }, $class;
  my $name = $self->{name};
  $self->set($self->{gear}->$name || '');
  return $self;
}

sub set {
  $_[0]->{widget}->set_text($_[1]);
}

sub save {
  my $self = shift;
  my $name = $self->{name};
  $self->{gear}->$name($self->{widget}->get_text);
};


package Pstax::GtkAutoDialog::CheckButton;

use Glib qw(TRUE FALSE);

sub new {
  my $class = shift;

  my $self = bless { @_ }, $class;
  my $name = $self->{name};
  $self->{widget}->set_active($self->{gear}->$name || FALSE);
  return $self;
}


sub save {
  my $self = shift;
  my $name = $self->{name};
  $self->{gear}->$name($self->{widget}->get_active);
};


package Pstax::GtkAutoDialog::TextView;


sub new {
  my $class = shift;

  my $self = bless { @_ }, $class;
  my $name = $self->{name};
  my $buffer = $self->{widget}->get_buffer;
  $buffer->set_text($self->{gear}->$name || '');
  return $self;
}


sub save {
  my $self = shift;
  my $name = $self->{name};
  my $buffer = $self->{widget}->get_buffer;
  my $s      = $buffer->get_start_iter;
  my $e      = $buffer->get_end_iter;
  my $text   = $buffer->get_text($s, $e, 0);
  $self->{gear}->$name($text);
};


1;
